%  General polynomial routines (not characteristic dependent but
% using internal polynomial structure).
%  Should be compiled/loaded ("linked") BEFORE characteristic
% dependent routines files.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Copyright (C) 1994,1995,1996,1997,1998 Joergen Backelin
%%
%% Bergman is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY.  No author or distributor
%% accepts responsibility to anyone for the consequences of using it
%% or for whether it serves any particular purpose or works at all,
%% unless (s)he says so in writing.  Refer to the Bergman General
%% Public License for full details.

%% Everyone is granted permission to copy, modify and redistribute
%% bergman, but only under the conditions described in the
%% Bergman General Public License.   A copy of this license is
%% supposed to have been given to you along with bergman so you
%% can know your rights and responsibilities.  It should be in a
%% file named copyright.  Among other things, the copyright notice
%% and this notice must be preserved on all copies.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


(OFF RAISE)

% CHANGES:

%  TermInsert now handles also terms with coinciding monomials.
% /JoeB 1998-07-05

%  Improved documentation; introduced typing; added QPOLMONMULT;
% corrected augredand to augredor in REDANDMONMULT./JoeB 1997-02-14

%  Made SubtractRedor non-returning./JoeB 1996-07-04

%  Brought TermInsert from monom.sl.  Added REDANDMONMULT.
%  Improved OrdinaryConcPolMonMult argument name mnemonics./JoeB 1996-06-21

% CREATED 1994-11-14 from part of the old (version 0.8) groebreductions.sl.


% Import: MonTimes, SubtractRedor1, MONLESSP, MONTIMES2, Num!&Den2QPol,
%	  PolNum, PolDen.
% Export: OrdinaryConcPolMonMult, SubtractRedor, TermInsert.

% Available: REDANDMONMULT, QPOLMONMULT, REDAND2QPOL.


% POLYNOMIAL ARITHMETICS ROUTINES. Most of them physically changes one
% argument, the redand. Some of them have rather arbitrary outputs, which
% should not be used! "Returns." below is short for "Returns what the
% mnemonic name indicates." No return indication = undefined output.


%	Changes returnstart by concatenating monquot*ppol (which may mean
%	monquot1*pol*monquot1). ppol may be a pure reductand or reductor
%	polynomial; returnstart should be the position of the last pair
%	in a polynomial of the same kind.

%# OrdinaryConcPolMonMult (purepol??position??, purepol, monquot) : - ;
(DE OrdinaryConcPolMonMult (returnstart ppol monquot)
 (PROG	(aa bb)
	(SETQ aa ppol)
	(SETQ bb returnstart)
  Ml	(ConcPol bb (Cf!&Mon2Pol (Lc aa) (MonTimes (Lpmon aa) monquot)))
	(COND ((PolDecap aa) (PolDecap bb) (GO Ml)))
 ))


%	Changes augredor.
%	cPol ("current polynomial") is to be reduced by, if its LeadMon augmon
%	is found on augredor. (cPol must be a Gbe, so that Mpt(augmon)=cPol.)

%# SubtractRedor (augredor, augmon) : - ;
(DE SubtractRedor (augredor augmon)
 (PROG	(aa)
	(SETQ aa augredor)
  Ml	(COND ((Mon!= (Lm (PolTail aa)) augmon)
	       (RETURN (SubtractRedor1 augredor aa (PPol (Mpt augmon)))))
	      ((AND (MONLESSP (PMon augmon) (Lpmon (PolTail aa)))
		   (Pol!? (PolTail (PolDecap aa))))
	       (GO Ml)))
 ))



%  Insert a coefficient-monomial (term) in decreasing order.
% 98-07-05: Should now work for Mon(cfmon) equal to occurring one;
%	    whence error check removed)/JoeB

%# TermInsert (redandterm, augredand) : redandposition ;
(DE TermInsert (cfmon augpol)
 (PROG	(aa bb)
	(SETQ aa (Tm2PMon cfmon))
	(SETQ bb augpol)
  Ml	(COND ((Pol0? (PolTail bb))
	       (RETURN (ConcPol bb (Tm2Pol cfmon))))
	      ((Mon!= (Lm (PolTail bb)) (Mon cfmon))
	       (COND ((SETQ aa (REDANDCF!+ (Cf cfmon) (Lc (PolTail bb))))
		      (PutLc (PolTail bb) aa))
		     (T
		      (RemoveNextTm bb)))
	       (RETURN bb))
	      ((MONLESSP aa (Lpmon (PolTail bb)))
	       (PolDecap bb)
	       (GO Ml)))
	(RETURN (InsertTm bb cfmon)) ))


%	Returns the product of augredand and pmon (in this order).
%	Non-destructive.
%	Meaning of prog variables: Return; Return position;
%	Input position.

%# REDANDMONMULT (augredand, puremon) : augredand ;
(DE REDANDMONMULT (augredand pmon)
 (PROG	(rt rp ip)
	(SETQ rp (SETQ rt (mkAPol)))
	(SETQ ip (PPol augredand))
	Ml
	(ConcPol rp (Cf!&Mon2Pol (Lc ip)
				 (MONTIMES2 (Lpmon ip) pmon)))
	(COND ((PolDecap ip)
	       (PolDecap rp)
	       (GO Ml)))
	(RETURN rt) ))


%	Returns the product of qpol and puremon (in this order).
%	Non-destructive.
%	Meaning of prog variables: Return; Return position;
%	Input position.

%# QPOLMONMULT (qpol, puremon) : qpol ;
(DE QPOLMONMULT (qpol pmon)
 (PROG	(rt rp ip)
	(SETQ rp (SETQ rt (Num!&Den2QPol (PolNum qpol) (PolDen qpol))))
	(SETQ ip (PPol qpol))
  Ml	(ConcPol rp (Cf!&Mon2Pol (Lc ip)
				 (MONTIMES2 (Lpmon ip) pmon)))
	(COND ((PolDecap ip)
	       (PolDecap rp)
	       (GO Ml)))
	(RETURN rt) ))


%  Not so smart procedure. We might wish to extract the content in some
% coefficient modes. Highly modularised, though.
%# REDAND2QPOL (redand) : qpol ;
(DE REDAND2QPOL (augpol)
 (PROG	(rt)
	(SETQ rt (Num!&Den2QPol REDANDCOEFFONE REDANDCOEFFONE))
	(ConcPol rt (CopyPPol (PPol augpol)))
	(RETURN rt) ))


(ON RAISE)
