%  The reduction procedures, used in the Buchberger algorithm
%  or in alternatives/variants.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Copyright (C) 1992,1994,199,1997 Joergen Backelin
%%
%% Bergman is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY.  No author or distributor
%% accepts responsibility to anyone for the consequences of using it
%% or for whether it serves any particular purpose or works at all,
%% unless (s)he says so in writing.  Refer to the Bergman General
%% Public License for full details.

%% Everyone is granted permission to copy, modify and redistribute
%% bergman, but only under the conditions described in the
%% Bergman General Public License.   A copy of this license is
%% supposed to have been given to you along with bergman so you
%% can know your rights and responsibilities.  It should be in a
%% file named copyright.  Among other things, the copyright notice
%% and this notice must be preserved on all copies.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


(OFF RAISE)

% CHANGES:

%  Improved documentation; added type documentation; added NORMALFORM;
% removed FindGroebF from interface procedure lists/JoeB 1997-02-14

%  Removed spurious (re-)definitions of PreciseniMonQuotient/JoeB 1996-08-06

% CREATED 1994-05-18 from part of the old (version 0.8) groebreductions.sl.
%
% The main changes from the old version are that the polynomial
% (coefficient depending) procedures are moved to a new file polynom.sl,
% and that the renewed reduction procedures are coefficient insensitives.

% Further changes:
%     Changed Cf* to Cf!*, Pol0? to Pol0!?, (EQ ... 1) to (Cf1!? ...),
%    ZEROP to Cf0!?, 1 to (Cf1), 0 to (Cf0).
%     Moved the PreciseniMonQuotient handling to the monomials units.


% Import: Redand2Redor, SubtractcRedor, SubtractRedand1, RedorMonMult,
%	  ReducePolStep, MonTimes, MONLESSP, niMonQuotient,
%	  SubtractQPol1, NormalFormStep, PreenQPol;
%	  GBasis, NOOFSPOLCALCS, !*IMMEDIATEFULLREDUCTION, MONONE.
% Export: ReducePol, FormSPol;
%	  NOOFNILREDUCTIONS.
% Available: NORMALFORM.

(GLOBAL '(GBasis NOOFSPOLCALCS !*IMMEDIATEFULLREDUCTION
	  NOOFNILREDUCTIONS MONONE))

% This is the module for the reduction routines. It uses a limited number
% of (tailored) polynomial routines, and some monomial ones. It is
% (hopefully) completely independent of coefficient and monomial modes.

% Recall the mnemonics: Pol=polynomial (augmented or not); N=number;
% Mon=monomial (with or without pointer); GCD=greatest common divisor;
% the rest is hopefully self-explanatory. As arguments, apol/pol or mon/pmon
% will tell what kind of Pol or Mon, respectively, is intended.
% Recall that polynomial and numerical inputs MUST be non-zero. (Monomials
% are by definition monic; the monomial 1 is allowed.)
% The mnemonic and argument order is "reverse lexicographic: p,n,m", i.e.,
% Polynomial, Number, Monomial.


% MAIN REDUCTIONS (exported):

%	Changes augredand. Returns.

% Changes its argument to a non-zero constant times the normal form
% of its input (with respect to GBasis). Returns NIL if the normal
% form is zero, its changed argument else.

% Note that the result is ASSOCIATED TO a normal form of the input,
% but NOT necessarily EQUAL TO one.

%#  ReducePol (augredand) : genaugredand ;
(DE ReducePol (augredand)
 (PROG	(AA BB CC)
	(COND ((APol0!? (SETQ AA augredand))
	       (COND ((NUMBERP NOOFNILREDUCTIONS)
		      (SETQ NOOFNILREDUCTIONS (ADD1 NOOFNILREDUCTIONS))))
	       (RETURN NIL)))
  Ml	(COND ((NOT (Mpt!? (Lm (PolTail AA))))
	       (PutMpt (Lm (PolTail AA))
		       (OR (FindGroebF (Lpmon (PolTail AA))) T))))
	(COND ((EQ (Mpt (Lm (PolTail AA))) T)
	       (PolDecap AA))
	      ((Mon!= (Lm (PolTail AA))
		      (Lm (SETQ CC (PPol (Mpt (Lm (PolTail AA)))))))
	       (COND ((OR !*IMMEDIATEFULLREDUCTION (EQ AA augredand))
		      (SubtractRedand1 augredand AA CC))
		     (T (PolDecap AA))))
	      (T
	       (ReducePolStep augredand
		      AA
		      CC 
		      (niMonQuotient (Lpmon (PolTail AA))
				     (Lpmon CC)))))
	(COND ((PolTail AA)
	       (GO Ml))
	      ((PPol!? augredand)
	       (Redand2Redor (PPol augredand))
	       (RETURN augredand)))
 ))


%	Changes qpol. Returns.

% Changes its argument to the normal form of its input (with respect
% to GBasis). Returns NIL if the normal form is zero, its changed
% argument else.

%# NORMALFORM (qpol) : genqpol ;
(DE NORMALFORM (qpol)
 (PROG	(AA BB CC)
	(COND ((APol0!? (SETQ AA qpol))
%	       (COND ((NUMBERP NOOFNILREDUCTIONS)
%		      (SETQ NOOFNILREDUCTIONS (ADD1 NOOFNILREDUCTIONS))))
	       (RETURN NIL))
	      ((Mon!= (QPol2Lm qpol) MONONE)
	       (RETURN qpol)))
  Ml	(COND ((NOT (Mpt!? (Lm (PolTail AA))))
	       (PutMpt (Lm (PolTail AA))
		       (OR (FindGroebF (Lpmon (PolTail AA))) T))))
	(COND ((EQ (Mpt (Lm (PolTail AA))) T)
	       (PolDecap AA))
	      ((Mon!= (Lm (PolTail AA))
		      (Lm (SETQ CC (PPol (Mpt (Lm (PolTail AA)))))))
	       (SubtractQPol1 qpol AA CC))
	      (T
	       (NormalFormStep qpol
		      AA
		      CC 
		      (niMonQuotient (Lpmon (PolTail AA))
				     (Lpmon CC)))))
	(COND ((PolTail AA)
	       (GO Ml))
	      ((PPol!? qpol)
	       (RETURN (PreenQPol qpol))))
 ))


%	Returns.

%# FormSPol (augmon, augmon, augmon, ?) : genaugredand ;
(DE FormSPol (mon1 mon2 mon3 placeno)
 (PROG	(RT)
	(COND ((NUMBERP NOOFSPOLCALCS)
	       (SETQ NOOFSPOLCALCS (ADD1 NOOFSPOLCALCS))))
	(SETQ RT (RedorMonMult (Mpt mon2)
			       (PreciseniMonQuotient (PMon mon1) (PMon mon2) 0)))
	(ReducePolStep RT
		       RT
		       (PPol (Mpt mon3))
		       (PreciseniMonQuotient (PMon mon1) (PMon mon3) placeno))
	(COND ((PPol!? RT)
	       (RETURN RT))) ))


%	Auxiliary. (Or should perhaps still be exported?)

%# FindGroebF (puremon) : genaugredor ;
(DE FindGroebF (pmon)
 (PROG	(AA)
	(COND ((NOT (SETQ AA (CDR GBasis))) (RETURN NIL)))
  Ml	(COND ((MONFACTORP (PMon (CAR AA)) pmon)
	       (RETURN (Mpt (CAR AA))))
	      ((SETQ AA (CDR AA))
	       (GO Ml))) ))


(ON RAISE)
